@@ -1,12 +1,8 @@
-"""
-Download Handler
-================
-"""
-
 import datetime
 import gzip
 import logging
 import os
+import re
 import sys
 import tempfile
 import threading
@@ -206,6 +202,22 @@ class DownloadHandler(ABC):
         for i in range(0, len(lst), number):
             yield lst[i : i + number]
 
+    def clean_keys(self, data):
+        """
+        Recursively clean dictionary keys for MongoDB by replacing illegal characters with hyphens
+        """
+        if isinstance(data, dict):
+            # Only clean dictionary keys and recurse for the values
+            return {
+                re.sub(r"[.$\s\+\!\*\&\?]", "-", key): self.clean_keys(value)
+                for key, value in data.items()
+            }
+        elif isinstance(data, list):
+            # For lists, just apply clean_keys on items
+            return [self.clean_keys(item) for item in data]
+        else:
+            return data
+
     def _db_bulk_writer(self, batch: list, initialization_run: bool = False):
         """
         Method to act as worker for writing queued entries into the database
@@ -219,11 +231,11 @@ class DownloadHandler(ABC):
             ):
                 # cves or cpe process item could yield None; so filter None from batch list
                 self.database[self.feed_type.lower()].insert_many(
-                    [x for x in batch if x is not None], ordered=False
+                    [self.clean_keys(x) for x in batch if x is not None], ordered=False
                 )
             else:
                 self.database[self.feed_type.lower()].bulk_write(
-                    [x for x in batch if x is not None], ordered=False
+                    [self.clean_keys(x) for x in batch if x is not None], ordered=False
                 )
         except BulkWriteError as err:
             self.logger.debug(f"Error during bulk write: {err}")
@@ -243,12 +255,26 @@ class DownloadHandler(ABC):
         """
         wd = tempfile.mkdtemp()
         filename = None
+        extension = None
+
+        # If the content type is generic and, therefore, cannot be used for determining
+        # the file type, extract the file extension from the url, instead.
+        if (
+            content_type == "application/octet-stream"
+            or content_type == "binary/octet-stream"
+            or content_type == "text/plain"
+        ):
+            extension = url.rsplit(".", 1)[1]
+            self.logger.debug(
+                f"Content-Type generic '{content_type}'; determining file type based on extension '.{extension}'"
+            )
 
         if (
             content_type == "application/zip"
             or content_type == "application/x-zip"
             or content_type == "application/x-zip-compressed"
             or content_type == "application/zip-compressed"
+            or extension == "zip"
         ):
             filename = os.path.join(wd, url.split("/")[-1][:-4])
             self.logger.debug(f"Saving file to: {filename}")
@@ -261,6 +287,7 @@ class DownloadHandler(ABC):
             or content_type == "application/gzip"
             or content_type == "application/x-gzip-compressed"
             or content_type == "application/gzip-compressed"
+            or extension == "gz"
         ):
             filename = os.path.join(wd, url.split("/")[-1][:-3])
             self.logger.debug(f"Saving file to: {filename}")
@@ -273,6 +300,8 @@ class DownloadHandler(ABC):
             content_type == "application/json"
             or content_type == "application/xml"
             or content_type == "text/xml"
+            or extension == "json"
+            or extension == "xml"
         ):
             filename = os.path.join(wd, url.split("/")[-1])
             self.logger.debug(f"Saving file to: {filename}")
