@@ -2,6 +2,7 @@ import datetime
 import glob
 import hashlib
 import json
+import re
 import shutil
 import time
 from typing import Any, Tuple
@@ -99,7 +100,7 @@ class CPEDownloads(NVDApiHandler):
 
         return cpe
 
-    def process_downloads(self, sites: list | None = None):
+    def process_downloads(self, sites: list | None = None, manual_days: int = 0):
         """
         Method to download and process files
         """
@@ -156,30 +157,41 @@ class CPEDownloads(NVDApiHandler):
                                     f"Retrieval of api data on url: {data_list.args[0]} failed...."
                                 )
             else:
-                last_mod_start_date = self.database[self.feed_type.lower()].find_one(
-                    {}, {"lastModified": 1}, sort=[("lastModified", -1)]
-                )
-
-                if last_mod_start_date is not None:
-                    if "lastModified" in last_mod_start_date:
-                        last_mod_start_date = last_mod_start_date[
-                            "lastModified"
-                        ] + datetime.timedelta(
-                            0, 1
-                        )  # add one second to prevent false results...
-                    else:
-                        raise KeyError(
-                            "Missing field 'lastModified' from database query..."
-                        )
-                else:
-                    self.logger.warning(
-                        "No records found in the mongodb cpe collection.."
-                    )
-                    return
-
-                    # Get datetime from runtime
+                # Get datetime from runtime
                 last_mod_end_date = datetime.datetime.now()
 
+                # Use configured day interval or detect from the latest entry in the database
+                if manual_days > 120:
+                    self.logger.warning(
+                        f"Update interval over 120 days not supported by the NVD API; ignoring"
+                    )
+                if manual_days > 0 and manual_days <= 120:
+                    last_mod_start_date = last_mod_end_date - datetime.timedelta(
+                        days=manual_days
+                    )
+                else:
+                    last_mod_start_date = self.database[
+                        self.feed_type.lower()
+                    ].find_one({}, {"lastModified": 1}, sort=[("lastModified", -1)])
+
+                    if last_mod_start_date is not None:
+                        if "lastModified" in last_mod_start_date:
+                            last_mod_start_date = last_mod_start_date[
+                                "lastModified"
+                            ] + datetime.timedelta(
+                                0, 1
+                            )  # add one second to prevent false results...
+                        else:
+                            raise KeyError(
+                                "Missing field 'lastModified' from database query..."
+                            )
+                    else:
+                        self.logger.warning(
+                            "No records found in the mongodb cpe collection.."
+                        )
+                        return
+                self.logger.info(f"Retrieving CPEs starting from {last_mod_start_date}")
+
                 try:
                     total_results = self.api_handler.get_count(
                         self.api_handler.datasource.CPE,
@@ -231,10 +243,10 @@ class CPEDownloads(NVDApiHandler):
             f"Duration: {datetime.timedelta(seconds=time.time() - start_time)}"
         )
 
-    def update(self, **kwargs):
+    def update(self, manual_days: int = 0):
         self.logger.info("CPE database update started")
 
-        self.process_downloads()
+        self.process_downloads(manual_days=manual_days)
 
         # if collection is non-existent; assume it's not an update
         if self.feed_type.lower() not in self.getTableNames():
@@ -374,158 +386,442 @@ class CVEDownloads(NVDApiHandler):
             return None
 
         cve = {
-            "id": item["cve"]["id"],
-            "assigner": item["cve"]["sourceIdentifier"],
-            "status": item["cve"]["vulnStatus"],
-            "published": parse_datetime(item["cve"]["published"], ignoretz=True),
-            "modified": parse_datetime(item["cve"]["lastModified"], ignoretz=True),
-            "lastModified": parse_datetime(item["cve"]["lastModified"], ignoretz=True),
+            "id": self.safe_get(item, "cve.id"),
+            "assigner": self.safe_get(item, "cve.sourceIdentifier"),
+            "status": self.safe_get(item, "cve.vulnStatus"),
+            "published": (
+                parse_datetime(self.safe_get(item, "cve.published"), ignoretz=True)
+                if self.safe_get(item, "cve.published")
+                else None
+            ),
+            "modified": (
+                parse_datetime(self.safe_get(item, "cve.lastModified"), ignoretz=True)
+                if self.safe_get(item, "cve.lastModified")
+                else None
+            ),
+            "lastModified": (
+                parse_datetime(self.safe_get(item, "cve.lastModified"), ignoretz=True)
+                if self.safe_get(item, "cve.lastModified")
+                else None
+            ),
         }
 
-        for description in item["cve"]["descriptions"]:
+        for description in self.safe_get(item, "cve.descriptions"):
             if description["lang"] == "en":
                 if "summary" in cve:
                     cve["summary"] += f" {description['value']}"
                 else:
                     cve["summary"] = description["value"]
 
-        if "metrics" in item["cve"]:
+        if "metrics" in self.safe_get(item, "cve"):
             cve["access"] = {}
             cve["impact"] = {}
-            if "cvssMetricV31" in item["cve"]["metrics"]:
+            if "cvssMetricV40" in self.safe_get(item, "cve.metrics"):
+                cve["impact4"] = {}
+                cve["exploitability4"] = {}
+                cve["impact4"]["vulnerable_system_confidentiality"] = self.safe_get(
+                    item,
+                    "cve.metrics.cvssMetricV40.[0].cvssData.vulnConfidentialityImpact",
+                )
+                cve["impact4"]["vulnerable_system_integrity"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].cvssData.vulnIntegrityImpact"
+                )
+                cve["impact4"]["vulnerable_system_availability"] = self.safe_get(
+                    item,
+                    "cve.metrics.cvssMetricV40.[0].cvssData.vulnAvailabilityImpact",
+                )
+                cve["impact4"]["subsequent_system_confidentiality"] = self.safe_get(
+                    item,
+                    "cve.metrics.cvssMetricV40.[0].cvssData.subConfidentialityImpact",
+                )
+                cve["impact4"]["subsequent_system_integrity"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].cvssData.subIntegrityImpact"
+                )
+                cve["impact4"]["subsequent_system_availability"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].cvssData.subAvailabilityImpact"
+                )
+                cve["impact4"]["attackvector"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].cvssData.attackVector"
+                )
+                cve["exploitability4"]["attackcomplexity"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].cvssData.attackComplexity"
+                )
+                cve["exploitability4"]["attackrequirements"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].cvssData.attackRequirements"
+                )
+                cve["exploitability4"]["privilegesrequired"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].cvssData.privilegesRequired"
+                )
+                cve["exploitability4"]["userinteraction"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].cvssData.userInteraction"
+                )
+                cve["exploitability4"]["exploitmaturity"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].cvssData.exploitMaturity"
+                )
+                cve["cvss4"] = (
+                    float(
+                        self.safe_get(
+                            item, "cve.metrics.cvssMetricV40.[0].cvssData.baseScore"
+                        )
+                    )
+                    if self.safe_get(
+                        item, "cve.metrics.cvssMetricV40.[0].cvssData.baseScore"
+                    )
+                    else None
+                )
+                cve["cvss4Vector"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].cvssData.vectorString"
+                )
+                cve["cvss4Time"] = (
+                    parse_datetime(
+                        self.safe_get(item, "cve.lastModified"), ignoretz=True
+                    )
+                    if self.safe_get(item, "cve.lastModified")
+                    else None
+                )
+                cve["cvss4Type"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].type"
+                )
+                cve["cvss4Source"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV40.[0].source"
+                )
+            else:
+                cve["cvss4"] = None
+
+            if "cvssMetricV31" in self.safe_get(item, "cve.metrics"):
                 cve["impact3"] = {}
                 cve["exploitability3"] = {}
-                cve["impact3"]["availability"] = item["cve"]["metrics"][
-                    "cvssMetricV31"
-                ][0]["cvssData"]["availabilityImpact"]
-                cve["impact3"]["confidentiality"] = item["cve"]["metrics"][
-                    "cvssMetricV31"
-                ][0]["cvssData"]["confidentialityImpact"]
-                cve["impact3"]["integrity"] = item["cve"]["metrics"]["cvssMetricV31"][
-                    0
-                ]["cvssData"]["integrityImpact"]
-                cve["exploitability3"]["attackvector"] = item["cve"]["metrics"][
-                    "cvssMetricV31"
-                ][0]["cvssData"]["attackVector"]
-                cve["exploitability3"]["attackcomplexity"] = item["cve"]["metrics"][
-                    "cvssMetricV31"
-                ][0]["cvssData"]["attackComplexity"]
-                cve["exploitability3"]["privilegesrequired"] = item["cve"]["metrics"][
-                    "cvssMetricV31"
-                ][0]["cvssData"]["privilegesRequired"]
-                cve["exploitability3"]["userinteraction"] = item["cve"]["metrics"][
-                    "cvssMetricV31"
-                ][0]["cvssData"]["userInteraction"]
-                cve["exploitability3"]["scope"] = item["cve"]["metrics"][
-                    "cvssMetricV31"
-                ][0]["cvssData"]["scope"]
-                cve["cvss3"] = float(
-                    item["cve"]["metrics"]["cvssMetricV31"][0]["cvssData"]["baseScore"]
+                cve["impact3"]["availability"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].cvssData.availabilityImpact"
                 )
-                cve["cvss3Vector"] = item["cve"]["metrics"]["cvssMetricV31"][0][
-                    "cvssData"
-                ]["vectorString"]
-                cve["impactScore3"] = float(
-                    item["cve"]["metrics"]["cvssMetricV31"][0]["impactScore"]
+                cve["impact3"]["confidentiality"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].cvssData.confidentialityImpact"
                 )
-                cve["exploitabilityScore3"] = float(
-                    item["cve"]["metrics"]["cvssMetricV31"][0]["exploitabilityScore"]
+                cve["impact3"]["integrity"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].cvssData.integrityImpact"
                 )
-                cve["cvss3Time"] = parse_datetime(
-                    item["cve"]["lastModified"], ignoretz=True
+                cve["exploitability3"]["attackvector"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].cvssData.attackVector"
                 )
-                cve["cvss3Type"] = item["cve"]["metrics"]["cvssMetricV31"][0]["type"]
-                cve["cvss3Source"] = item["cve"]["metrics"]["cvssMetricV31"][0][
-                    "source"
-                ]
-            elif "cvssMetricV30" in item["cve"]["metrics"]:
+                cve["exploitability3"]["attackcomplexity"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].cvssData.attackComplexity"
+                )
+                cve["exploitability3"]["privilegesrequired"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].cvssData.privilegesRequired"
+                )
+                cve["exploitability3"]["userinteraction"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].cvssData.userInteraction"
+                )
+                cve["exploitability3"]["scope"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].cvssData.scope"
+                )
+                cve["cvss3"] = (
+                    float(
+                        self.safe_get(
+                            item, "cve.metrics.cvssMetricV31.[0].cvssData.baseScore"
+                        )
+                    )
+                    if self.safe_get(
+                        item, "cve.metrics.cvssMetricV31.[0].cvssData.baseScore"
+                    )
+                    else None
+                )
+                cve["cvss3Vector"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].cvssData.vectorString"
+                )
+                cve["impactScore3"] = (
+                    float(
+                        self.safe_get(item, "cve.metrics.cvssMetricV31.[0].impactScore")
+                    )
+                    if self.safe_get(item, "cve.metrics.cvssMetricV31.[0].impactScore")
+                    else None
+                )
+                cve["exploitabilityScore3"] = (
+                    float(
+                        self.safe_get(
+                            item, "cve.metrics.cvssMetricV31.[0].exploitabilityScore"
+                        )
+                    )
+                    if self.safe_get(
+                        item, "cve.metrics.cvssMetricV31.[0].exploitabilityScore"
+                    )
+                    else None
+                )
+                cve["cvss3Time"] = (
+                    parse_datetime(
+                        self.safe_get(item, "cve.lastModified"), ignoretz=True
+                    )
+                    if self.safe_get(item, "cve.lastModified")
+                    else None
+                )
+                cve["cvss3Type"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].type"
+                )
+                cve["cvss3Source"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV31.[0].source"
+                )
+            elif "cvssMetricV30" in self.safe_get(item, "cve.metrics"):
                 cve["impact3"] = {}
                 cve["exploitability3"] = {}
-                cve["impact3"]["availability"] = item["cve"]["metrics"][
-                    "cvssMetricV30"
-                ][0]["cvssData"]["availabilityImpact"]
-                cve["impact3"]["confidentiality"] = item["cve"]["metrics"][
-                    "cvssMetricV30"
-                ][0]["cvssData"]["confidentialityImpact"]
-                cve["impact3"]["integrity"] = item["cve"]["metrics"]["cvssMetricV30"][
-                    0
-                ]["cvssData"]["integrityImpact"]
-                cve["exploitability3"]["attackvector"] = item["cve"]["metrics"][
-                    "cvssMetricV30"
-                ][0]["cvssData"]["attackVector"]
-                cve["exploitability3"]["attackcomplexity"] = item["cve"]["metrics"][
-                    "cvssMetricV30"
-                ][0]["cvssData"]["attackComplexity"]
-                cve["exploitability3"]["privilegesrequired"] = item["cve"]["metrics"][
-                    "cvssMetricV30"
-                ][0]["cvssData"]["privilegesRequired"]
-                cve["exploitability3"]["userinteraction"] = item["cve"]["metrics"][
-                    "cvssMetricV30"
-                ][0]["cvssData"]["userInteraction"]
-                cve["exploitability3"]["scope"] = item["cve"]["metrics"][
-                    "cvssMetricV30"
-                ][0]["cvssData"]["scope"]
-                cve["cvss3"] = float(
-                    item["cve"]["metrics"]["cvssMetricV30"][0]["cvssData"]["baseScore"]
+                cve["impact3"]["availability"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV30.[0].cvssData.availabilityImpact"
                 )
-                cve["cvss3Vector"] = item["cve"]["metrics"]["cvssMetricV30"][0][
-                    "cvssData"
-                ]["vectorString"]
-                cve["impactScore3"] = float(
-                    item["cve"]["metrics"]["cvssMetricV30"][0]["impactScore"]
+                cve["impact3"]["confidentiality"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV30.[0].cvssData.confidentialityImpact"
                 )
-                cve["exploitabilityScore3"] = float(
-                    item["cve"]["metrics"]["cvssMetricV30"][0]["exploitabilityScore"]
+                cve["impact3"]["integrity"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV30.[0].cvssData.integrityImpact"
                 )
-                cve["cvss3Time"] = parse_datetime(
-                    item["cve"]["lastModified"], ignoretz=True
+                cve["exploitability3"]["attackvector"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV30.[0].cvssData.attackVector"
+                )
+                cve["exploitability3"]["attackcomplexity"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV30.[0].cvssData.attackComplexity"
+                )
+                cve["exploitability3"]["privilegesrequired"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV30.[0].cvssData.privilegesRequired"
+                )
+                cve["exploitability3"]["userinteraction"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV30.[0].cvssData.userInteraction"
+                )
+                cve["exploitability3"]["scope"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV30.[0].cvssData.scope"
+                )
+                cve["cvss3"] = (
+                    float(
+                        self.safe_get(
+                            item, "cve.metrics.cvssMetricV30.[0].cvssData.baseScore"
+                        )
+                    )
+                    if self.safe_get(
+                        item, "cve.metrics.cvssMetricV30.[0].cvssData.baseScore"
+                    )
+                    else None
+                )
+                cve["impactScore3"] = (
+                    float(
+                        self.safe_get(item, "cve.metrics.cvssMetricV30.[0].impactScore")
+                    )
+                    if self.safe_get(item, "cve.metrics.cvssMetricV30.[0].impactScore")
+                    else None
+                )
+                cve["exploitabilityScore3"] = (
+                    float(
+                        self.safe_get(
+                            item, "cve.metrics.cvssMetricV30.[0].exploitabilityScore"
+                        )
+                    )
+                    if self.safe_get(
+                        item, "cve.metrics.cvssMetricV30.[0].exploitabilityScore"
+                    )
+                    else None
+                )
+                cve["cvss3Time"] = (
+                    parse_datetime(
+                        self.safe_get(item, "cve.lastModified"), ignoretz=True
+                    )
+                    if self.safe_get(item, "cve.lastModified")
+                    else None
+                )
+                cve["cvss3Type"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV30.[0].type"
+                )
+                cve["cvss3Source"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV30.[0].source"
                 )
-                cve["cvss3Type"] = item["cve"]["metrics"]["cvssMetricV30"][0]["type"]
-                cve["cvss3Source"] = item["cve"]["metrics"]["cvssMetricV30"][0][
-                    "source"
-                ]
             else:
                 cve["cvss3"] = None
 
-            if "cvssMetricV2" in item["cve"]["metrics"]:
-                cve["access"]["authentication"] = item["cve"]["metrics"][
-                    "cvssMetricV2"
-                ][0]["cvssData"]["authentication"]
-                cve["access"]["complexity"] = item["cve"]["metrics"]["cvssMetricV2"][0][
-                    "cvssData"
-                ]["accessComplexity"]
-                cve["access"]["vector"] = item["cve"]["metrics"]["cvssMetricV2"][0][
-                    "cvssData"
-                ]["accessVector"]
-                cve["impact"]["availability"] = item["cve"]["metrics"]["cvssMetricV2"][
-                    0
-                ]["cvssData"]["availabilityImpact"]
-                cve["impact"]["confidentiality"] = item["cve"]["metrics"][
-                    "cvssMetricV2"
-                ][0]["cvssData"]["confidentialityImpact"]
-                cve["impact"]["integrity"] = item["cve"]["metrics"]["cvssMetricV2"][0][
-                    "cvssData"
-                ]["integrityImpact"]
-                cve["cvss"] = float(
-                    item["cve"]["metrics"]["cvssMetricV2"][0]["cvssData"]["baseScore"]
+            if "cvssMetricV2" in self.safe_get(item, "cve.metrics"):
+                cve["access"]["authentication"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV2.[0].cvssData.authentication"
                 )
-                cve["exploitabilityScore"] = float(
-                    item["cve"]["metrics"]["cvssMetricV2"][0]["exploitabilityScore"]
+                cve["access"]["complexity"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV2.[0].cvssData.accessComplexity"
                 )
-                cve["impactScore"] = float(
-                    item["cve"]["metrics"]["cvssMetricV2"][0]["impactScore"]
+                cve["access"]["vector"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV2.[0].cvssData.accessVector"
                 )
-                cve["cvssTime"] = parse_datetime(
-                    item["cve"]["lastModified"], ignoretz=True
+                cve["impact"]["availability"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV2.[0].cvssData.availabilityImpact"
+                )
+                cve["impact"]["confidentiality"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV2.[0].cvssData.confidentialityImpact"
+                )
+                cve["impact"]["integrity"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV2.[0].cvssData.integrityImpact"
+                )
+                cve["cvss"] = (
+                    float(
+                        self.safe_get(
+                            item, "cve.metrics.cvssMetricV2.[0].cvssData.baseScore"
+                        )
+                    )
+                    if self.safe_get(
+                        item, "cve.metrics.cvssMetricV2.[0].cvssData.baseScore"
+                    )
+                    else None
+                )
+                cve["exploitabilityScore"] = (
+                    float(
+                        self.safe_get(
+                            item, "cve.metrics.cvssMetricV2.[0].exploitabilityScore"
+                        )
+                    )
+                    if self.safe_get(
+                        item, "cve.metrics.cvssMetricV2.[0].exploitabilityScore"
+                    )
+                    else None
+                )
+                cve["impactScore"] = (
+                    float(
+                        self.safe_get(item, "cve.metrics.cvssMetricV2.[0].impactScore")
+                    )
+                    if self.safe_get(item, "cve.metrics.cvssMetricV2.[0].impactScore")
+                    else None
+                )
+                cve["cvssTime"] = (
+                    parse_datetime(
+                        self.safe_get(item, "cve.lastModified"), ignoretz=True
+                    )
+                    if self.safe_get(item, "cve.lastModified")
+                    else None
                 )  # NVD JSON lacks the CVSS time which was present in the original XML format
-                cve["cvssVector"] = item["cve"]["metrics"]["cvssMetricV2"][0][
-                    "cvssData"
-                ]["vectorString"]
-                cve["cvssType"] = item["cve"]["metrics"]["cvssMetricV2"][0]["type"]
-                cve["cvssSource"] = item["cve"]["metrics"]["cvssMetricV2"][0]["source"]
+                cve["cvssVector"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV2.[0].cvssData.vectorString"
+                )
+                cve["cvssType"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV2.[0].type"
+                )
+                cve["cvssSource"] = self.safe_get(
+                    item, "cve.metrics.cvssMetricV2.[0].source"
+                )
             else:
                 cve["cvss"] = None
 
+        cve["cvss_data"] = {"cvss2": {}, "cvss3": {}, "cvss4": {}}
+
+        for version in [
+            "cvssMetricV40",
+            "cvssMetricV31",
+            "cvssMetricV30",
+            "cvssMetricV2",
+        ]:
+            if version in self.safe_get(item, "cve.metrics"):
+                for metric in item["cve"]["metrics"][version]:
+                    cvss_key = (
+                        "cvss4"
+                        if version == "cvssMetricV40"
+                        else (
+                            "cvss3"
+                            if version in ["cvssMetricV31", "cvssMetricV30"]
+                            else "cvss2"
+                        )
+                    )
+                    source = self.safe_get(metric, "source")
+
+                    entry = {
+                        "type": self.safe_get(metric, "type"),
+                        "vectorString": self.safe_get(metric, "cvssData.vectorString"),
+                        "baseScore": self.safe_get(metric, "cvssData.baseScore"),
+                    }
+
+                    if cvss_key == "cvss4":
+                        entry.update(
+                            {
+                                "vulnerable_system_confidentiality": self.safe_get(
+                                    metric, "cvssData.vulnConfidentialityImpact"
+                                ),
+                                "vulnerable_system_integrity": self.safe_get(
+                                    metric, "cvssData.vulnIntegrityImpact"
+                                ),
+                                "vulnerable_system_availability": self.safe_get(
+                                    metric, "cvssData.vulnAvailabilityImpact"
+                                ),
+                                "subsequent_system_confidentiality": self.safe_get(
+                                    metric, "cvssData.subConfidentialityImpact"
+                                ),
+                                "subsequent_system_integrity": self.safe_get(
+                                    metric, "cvssData.subIntegrityImpact"
+                                ),
+                                "subsequent_system_availability": self.safe_get(
+                                    metric, "cvssData.subAvailabilityImpact"
+                                ),
+                                "attackVector": self.safe_get(
+                                    metric, "cvssData.attackVector"
+                                ),
+                                "attackComplexity": self.safe_get(
+                                    metric, "cvssData.attackComplexity"
+                                ),
+                                "attackRequirements": self.safe_get(
+                                    metric, "cvssData.attackRequirements"
+                                ),
+                                "privilegesRequired": self.safe_get(
+                                    metric, "cvssData.privilegesRequired"
+                                ),
+                                "userInteraction": self.safe_get(
+                                    metric, "cvssData.userInteraction"
+                                ),
+                                "exploitMaturity": self.safe_get(
+                                    metric, "cvssData.exploitMaturity"
+                                ),
+                            }
+                        )
+                    elif cvss_key == "cvss3":
+                        entry.update(
+                            {
+                                "confidentialityImpact": self.safe_get(
+                                    metric, "cvssData.confidentialityImpact"
+                                ),
+                                "integrityImpact": self.safe_get(
+                                    metric, "cvssData.integrityImpact"
+                                ),
+                                "availabilityImpact": self.safe_get(
+                                    metric, "cvssData.availabilityImpact"
+                                ),
+                                "attackVector": self.safe_get(
+                                    metric, "cvssData.attackVector"
+                                ),
+                                "attackComplexity": self.safe_get(
+                                    metric, "cvssData.attackComplexity"
+                                ),
+                                "privilegesRequired": self.safe_get(
+                                    metric, "cvssData.privilegesRequired"
+                                ),
+                                "userInteraction": self.safe_get(
+                                    metric, "cvssData.userInteraction"
+                                ),
+                                "scope": self.safe_get(metric, "cvssData.scope"),
+                            }
+                        )
+                    elif cvss_key == "cvss2":
+                        entry.update(
+                            {
+                                "authentication": self.safe_get(
+                                    metric, "cvssData.authentication"
+                                ),
+                                "accessComplexity": self.safe_get(
+                                    metric, "cvssData.accessComplexity"
+                                ),
+                                "accessVector": self.safe_get(
+                                    metric, "cvssData.accessVector"
+                                ),
+                                "confidentialityImpact": self.safe_get(
+                                    metric, "cvssData.confidentialityImpact"
+                                ),
+                                "integrityImpact": self.safe_get(
+                                    metric, "cvssData.integrityImpact"
+                                ),
+                                "availabilityImpact": self.safe_get(
+                                    metric, "cvssData.availabilityImpact"
+                                ),
+                            }
+                        )
+
+                    if source not in cve["cvss_data"][cvss_key]:
+                        cve["cvss_data"][cvss_key][source] = []
+                    cve["cvss_data"][cvss_key][source].append(entry)
+
         if "references" in item["cve"]:
             cve["references"] = []
             for ref in item["cve"]["references"]:
@@ -540,6 +836,8 @@ class CVEDownloads(NVDApiHandler):
             cve["vulnerable_configuration_stems"] = []
             cve["configurations"] = item["cve"]["configurations"]
             for node in item["cve"]["configurations"]:
+                if "nodes" not in node:
+                    continue
                 for cpe in node["nodes"]:
                     if "cpeMatch" in cpe:
                         for cpeuri in cpe["cpeMatch"]:
@@ -626,23 +924,33 @@ class CVEDownloads(NVDApiHandler):
                                     self.stem(cpeuri["criteria"]),
                                 )
         if "weaknesses" in item["cve"]:
+            cwe_set = set()
+
             for problem in item["cve"]["weaknesses"]:
-                for cwe in problem[
-                    "description"
-                ]:  # NVD JSON not clear if we can get more than one CWE per CVE (until we take the last one) -
-                    # NVD-CWE-Other??? list?
+                for cwe in problem.get("description", []):
                     if cwe["lang"] == "en":
-                        cve["cwe"] = cwe["value"]
-            if not ("cwe" in cve):
-                cve["cwe"] = defaultvalue["cwe"]
+                        cwe_set.add(cwe["value"])
+
+            cve["cwe"] = sorted(cwe_set)
+
+            # If at least one valid CWE exists, remove all "NVD-CWE-*" entries
+            if any(not re.match(r"^NVD-CWE-", cwe) for cwe in cve["cwe"]):
+                cve["cwe"] = [
+                    cwe for cwe in cve["cwe"] if not re.match(r"^NVD-CWE-", cwe)
+                ]
+
+            # If the list is empty after filtering, assign the default value
+            if not cve["cwe"]:
+                cve["cwe"] = [defaultvalue["cwe"]]
         else:
-            cve["cwe"] = defaultvalue["cwe"]
+            # Assign the default value if "weaknesses" is not present
+            cve["cwe"] = [defaultvalue["cwe"]]
 
         cve["vulnerable_configuration_cpe_2_2"] = []
 
         return cve
 
-    def process_downloads(self, sites: list = None):
+    def process_downloads(self, sites: list = None, manual_days: int = 0):
         """
         Method to download and process files
         """
@@ -699,26 +1007,41 @@ class CVEDownloads(NVDApiHandler):
                                     f"Retrieval of api data on url: {data_list.args[0]} failed...."
                                 )
             else:
-                last_mod_start_date = self.database[self.feed_type.lower()].find_one(
-                    {}, {"lastModified": 1}, sort=[("lastModified", -1)]
-                )
-
-                if last_mod_start_date is not None:
-                    if "lastModified" in last_mod_start_date:
-                        last_mod_start_date = last_mod_start_date["lastModified"]
-                    else:
-                        raise KeyError(
-                            "Missing field 'lastModified' from database query..."
-                        )
-                else:
-                    self.logger.warning(
-                        "No records found in the mongodb cves collection.."
-                    )
-                    return
-
-                    # Get datetime from runtime
+                # Get datetime from runtime
                 last_mod_end_date = datetime.datetime.now()
 
+                # Use configured day interval or detect from the latest entry in the database
+                if manual_days > 120:
+                    self.logger.warning(
+                        f"Update interval over 120 days not supported by the NVD API; ignoring"
+                    )
+                if manual_days > 0 and manual_days <= 120:
+                    last_mod_start_date = last_mod_end_date - datetime.timedelta(
+                        days=manual_days
+                    )
+                else:
+                    last_mod_start_date = self.database[
+                        self.feed_type.lower()
+                    ].find_one({}, {"lastModified": 1}, sort=[("lastModified", -1)])
+
+                    if last_mod_start_date is not None:
+                        if "lastModified" in last_mod_start_date:
+                            last_mod_start_date = last_mod_start_date[
+                                "lastModified"
+                            ] + datetime.timedelta(
+                                0, 1
+                            )  # add one second to prevent false results...
+                        else:
+                            raise KeyError(
+                                "Missing field 'lastModified' from database query..."
+                            )
+                    else:
+                        self.logger.warning(
+                            "No records found in the mongodb cpe collection.."
+                        )
+                        return
+                self.logger.info(f"Retrieving CVEs starting from {last_mod_start_date}")
+
                 try:
                     total_results = self.api_handler.get_count(
                         self.api_handler.datasource.CVE,
@@ -766,14 +1089,16 @@ class CVEDownloads(NVDApiHandler):
             # Set the last update time in the info collection
             self.setColUpdate(self.feed_type.lower(), self.last_modified)
 
+        self.log_statistics()
+
         self.logger.info(
             f"Duration: {datetime.timedelta(seconds=time.time() - start_time)}"
         )
 
-    def update(self):
+    def update(self, manual_days: int = 0):
         self.logger.info("CVE database update started")
 
-        self.process_downloads()
+        self.process_downloads(manual_days=manual_days)
 
         # if collection is non-existent; assume it's not an update
         if self.feed_type.lower() not in self.getTableNames():
@@ -969,22 +1294,20 @@ class CWEDownloads(XMLFileHandler):
     def file_to_queue(self, file_tuple: Tuple[str, str]):
         working_dir, filename = file_tuple
 
-        for f in glob.glob(f"{working_dir}/*.xml"):
-            filename = f
+        for filename in glob.glob(f"{working_dir}/*.xml"):
+            self.parser.parse(filename)
+            x = 0
+            for cwe in self.ch.cwe:
+                try:
+                    cwe["related_weaknesses"] = list(set(cwe["related_weaknesses"]))
+                    cwe["description"] = cwe["Description"]
+                    cwe.pop("Description")
+                except KeyError:
+                    pass
+                self.process_item(cwe)
+                x += 1
 
-        self.parser.parse(f"file://{filename}")
-        x = 0
-        for cwe in self.ch.cwe:
-            try:
-                cwe["related_weaknesses"] = list(set(cwe["related_weaknesses"]))
-                cwe["description"] = cwe["Description"]
-                cwe.pop("Description")
-            except KeyError:
-                pass
-            self.process_item(cwe)
-            x += 1
-
-        self.logger.debug(f"Processed {x} entries from file: {filename}")
+            self.logger.debug(f"Processed {x} entries from file: {filename}")
 
         try:
             self.logger.debug(f"Removing working dir: {working_dir}")
