@@ -38,6 +38,8 @@ class GenericDatabaseFactory(DatasourceConnection):
             "cves": [
                 "cvss",
                 "cvss3",
+                "cvss4",
+                "cvss_data",
                 "summary",
                 "vendors",
                 "products",
