@@ -1,8 +1,3 @@
-"""
-via4
-====
-"""
-
 from CveXplore.common.data_source_connection import DatasourceConnection
 
 
